/* 
 * File:   FunctionBind.h
 * Author: denis
 * 
 * Macros to bind boost::function to a function inside the class
 * Due to boost::bind being picky about the names of the functions (or maybe it's a compiler's problem),
 * we are going to bind to functions that have a name with the 1 at the end
 *
 * Created on January 24, 2012, 2:40 PM
 */

#ifndef FUNCTIONBIND_H
#define	FUNCTIONBIND_H

/// Delegate (or function pointer)
#define DELEGATE(CLASS,FUNC,OUTPUT,...) OUTPUT (CLASS::*FUNC) (__VA_ARGS__)
#define DELEGATEV(CLASS,FUNC,OUTPUT) DELEGATE(CLASS,FUNC,OUTPUT,)

// Delegates for standard functions: init, sizeofdata, print, readData, setBufferSource, resetBufferSource
// Have to intput the name of the delegate
#define Init_t(FUNC)              DELEGATEV(RawDAQData, FUNC, void    )
#define Sizeofdata_t(FUNC)        DELEGATEV(RawDAQData, FUNC, uint32_t) const
#define Print_t(FUNC)             DELEGATE(RawDAQData, FUNC, void    , std::ostream&) const
#define ReadData_t(FUNC)          DELEGATE(RawDAQData, FUNC, void*   , const void*  )
#define SetBufferSource_t(FUNC)   DELEGATE(RawDAQData, FUNC, bool    , const void*  )
#define ResetBufferSource_t(FUNC) DELEGATEV(RawDAQData, FUNC, bool    )

// No input name
#define init_t              Init_t()
#define sizeofdata_t        Sizeofdata_t()
#define print_t             Print_t()
#define readData_t          ReadData_t()
#define setBufferSource_t   SetBufferSource_t()
#define resetBufferSource_t ResetBufferSource_t()

/// Macro to execute the function given a function pointer
#define EXECUTE_GENERAL_FUNCTION(FULLFUNCTIONNAME) (this->*FULLFUNCTIONNAME)

// These macros are used in a deep copy of delegates
#define DEEPCOPY_FUNCTION(FUNC) _function_##FUNC(copyin._function_##FUNC)
#define DEEPCOPY_GET(FUNC)      DEEPCOPY_FUNCTION(get##FUNC)
#define DEEPCOPY_IS(FUNC)       DEEPCOPY_FUNCTION(is##FUNC)
#define DEEPCOPY_SET(FUNC)      DEEPCOPY_FUNCTION(set##FUNC)
#define DEEPCOPY_GET_AND_SET(FUNC)\
                DEEPCOPY_GET(FUNC)\
               ,DEEPCOPY_SET(FUNC)
#define DEEPCOPY_IS_AND_SET(FUNC)\
                DEEPCOPY_IS(FUNC)\
               ,DEEPCOPY_SET(FUNC)

// These methods are used for zeroing out delegates
#define ZEROOUT_FUNCTION(FUNC) _function_##FUNC(NULL)
#define ZEROOUT_GET(FUNC)      ZEROOUT_FUNCTION(get##FUNC)
#define ZEROOUT_IS(FUNC)       ZEROOUT_FUNCTION(is##FUNC)
#define ZEROOUT_SET(FUNC)      ZEROOUT_FUNCTION(set##FUNC)
#define ZEROOUT_GET_AND_SET(FUNC)\
                ZEROOUT_GET(FUNC)\
               ,ZEROOUT_SET(FUNC)
#define ZEROOUT_IS_AND_SET(FUNC)\
                ZEROOUT_IS(FUNC)\
               ,ZEROOUT_SET(FUNC)


#endif /* FUNCTIONBIND_H */
