#ifndef DAQRAWTRIGGER_H
#define DAQRAWTRIGGER_H
#include "DAQDataFormats/RawTriggerV2.h"
#include "DAQDataFormats/RawTriggerHeader.h"
#include "DAQDataFormats/RawTriggerRange.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawTrigger
#define  LAST_CLASS_VERSION           2
#define  CURRENT_FIELDS_SUB_NAMESPACE rawtrigger
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats {

    namespace CURRENT_FIELDS_SUB_NAMESPACE {
        enum  TriggerVersion {
                RAWTRIGGER_MAXIMUM_SIZE = 50 ///< needed to reserve the buffer
        };// end of NanoSliceVersion
    }// end of namespace rawnano
    

  //! Class to hold the data from the global trigger in the correct format
  /*!
    The RawTrigger class is designed to mimic the format of the actual
    data that emitted from the global trigger process.  This information
    include the source of the trigger, the trigger number etc...

    The class supports general initialization with an internal or external
    data buffer as well as copyin style initialization for use with
    STL containers.
  */
  class RawTrigger: public PREVIOUS_VERSION_CLASS_NAME {
  
    public:

        // Constructors
        RawTrigger (const version_t version=triggerheader::RAWTRIGGERHEADER_LATEST_VERSION); //! The default constructor
        RawTrigger (const RawTrigger& copy_in);    //! Deep copy constructor
        RawTrigger (const void* trigger_buffer, const version_t version = rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION ); //! Shallow copy constructor
        /// Destructor
        ~RawTrigger();

        IMPLEMENT_MAIN_STANDARD
        IMPLEMENT_MAIN_setBufferSource
	
        /// Method to read data from a PIPE (not socket).
        /// Only one read can be made. After that the pipe is closed.
        /// According to Ron Rechenmacher, the maximum size of a PIPE is 512 bytes.
	readDataStatus readData(const int fd){RETURN_FUNCTION(readDataFromFileDescriptor)(fd);}

        IMPLEMENT_SET_MAIN(TriggerHeader      , daqdataformats::RawTriggerHeader&)
        IMPLEMENT_SET_MAIN(TriggerTimingMarker, daqdataformats::RawTriggerTimingMarker&)
        IMPLEMENT_SET_MAIN(TriggerRange       , daqdataformats::RawTriggerRange&)
        IMPLEMENT_SET_MAIN(TriggerTime        , daqdataformats::RawTriggerTime&)
        IMPLEMENT_SET_MAIN(TriggerMask        , daqdataformats::RawTriggerMask&)
        IMPLEMENT_GET_AND_SET_MAIN(TriggerStart, uint64_t)


            IMPLEMENT_FIGUREOUT_VERSION_STATIC(RawTriggerHeader)
        private:
            bool setFunctionPointers();

        
  };
  
  
} // END NAMESPCE daqdataformats
#endif /* DAQRAWTRIGGER_H */
