#ifndef _DBUTILS_DAQCONFIG_GLOBALNAMESPEC_H
#define _DBUTILS_DAQCONFIG_GLOBALNAMESPEC_H

#include <string>

namespace dbutils {
namespace daqconfig {

/**
 * Simple container class to hold the identifiers that specify a particular
 * named global configuration.
 *
 * @author Kurt Biery
 * @version $Revision: 1.7 $ $Date: 2011/11/30 15:28:14 $
 */

class GlobalNameSpec {

 public:

    // ********************
    // *** Constructors ***
    // ********************

    GlobalNameSpec(const std::string& configName,
                   const std::string& dcmOperatingMode,
                   const std::string& detectorId);

    GlobalNameSpec(const std::string& configName,
                   const std::string& dcmOperatingMode,
                   const std::string& detectorId,
                   const std::string& timestamp);

    // *****************
    // *** Accessors ***
    // *****************

    std::string getConfigName() const {return _configName;}
    std::string getDCMOpMode() const {return _dcmOperatingMode;}
    std::string getDetectorId() const {return _detectorId;}
    std::string getDetectorName() const {return _detectorId;}

    bool hasTimestamp() const {return _hasTimestamp;}
    std::string getTimestamp() const {return _timestamp;}

    bool skipInvalidStateTest() const {return _skipInvalidStateTest;}
    bool isSlowControlsConfiguration() const {return _isSlowControlsConfig;}
    bool isDAQConfiguration() const {return (! _isSlowControlsConfig);}

    int getUsageCount() const {return _usageCount;}

    std::string toString(int overallIndent = 0) const;

    // *****************
    // *** Modifiers ***
    // *****************

    void skipInvalidStateTest(bool setting) {
        _skipInvalidStateTest = setting;
    }
    void isSlowControlsConfiguration(bool setting) {
        _isSlowControlsConfig = setting;
    }
    void isDAQConfiguration(bool setting) {
        _isSlowControlsConfig = (! setting);
    }
    void setUsageCount(int value) {_usageCount = value;}
    void setSortIndex(int value) {_sortIndex = value;}

    // *****************
    // *** Operators ***
    // *****************

    bool operator<(const GlobalNameSpec& other) const;

 private:

    void _addSpaces(std::string& workString, int count) const;

    std::string _configName;
    std::string _dcmOperatingMode;
    std::string _detectorId;

    bool _hasTimestamp;
    std::string _timestamp;

    bool _skipInvalidStateTest;
    bool _isSlowControlsConfig;

    int _usageCount;
    int _sortIndex;
};

};  // end of namespace daqconfig
};  // end of namespace dbutils

#endif
