#ifndef _DBUTILS_DAQCONFIG_IDSPEC_H
#define _DBUTILS_DAQCONFIG_IDSPEC_H

#include <string>
#include <stdint.h>

namespace dbutils {
namespace daqconfig {

/**
 * Simple container class to hold the ID-based parameters that
 * specify a particular configuration.
 *
 * @author Kurt Biery
 * @version $Revision: 1.1 $ $Date: 2011/11/28 15:15:12 $
 */

class IDSpec {

 public:

    enum CONFIG_LEVEL { GLOBAL, SUBSYSTEM };
    enum MAJOR_SYSTEM_TYPE { DAQ, SLOWCONTROLS };

    // ********************
    // *** Constructors ***
    // ********************

    explicit IDSpec(const int64_t& configId, CONFIG_LEVEL globalOrSubsytem,
                    MAJOR_SYSTEM_TYPE daqOrSlowControls);

    // *****************
    // *** Accessors ***
    // *****************

    int64_t getConfigId() const {return _configId;}

    bool isGlobalConfiguration() const {return (! _isSubsystemConfig);}
    bool isSubsystemConfiguration() const {return _isSubsystemConfig;}

    bool isDAQConfiguration() const {return (! _isSlowControlsConfig);}
    bool isSlowControlsConfiguration() const {return _isSlowControlsConfig;}

    // *****************
    // *** Modifiers ***
    // *****************

    void isGlobalConfiguration(bool setting) {
        _isSubsystemConfig = (! setting);
    }
    void isSubsystemConfiguration(bool setting) {
        _isSubsystemConfig = setting;
    }

    void isDAQConfiguration(bool setting) {
        _isSlowControlsConfig = (! setting);
    }
    void isSlowControlsConfiguration(bool setting) {
        _isSlowControlsConfig = setting;
    }

 private:

    int64_t _configId;
    bool _isSubsystemConfig;
    bool _isSlowControlsConfig;

};

};  // end of namespace daqconfig
};  // end of namespace dbutils

#endif
