#ifndef _DBUTILS_DCS_APDSETTINGS_H
#define _DBUTILS_DCS_APDSETTINGS_H

#include <DatabaseUtils/DAQConfig/ConfigDataTree.h>
#include <DatabaseUtils/DAQConfig/GlobalNameSpec.h>
#include <DatabaseUtils/DAQConfig/SubsystemNameSpec.h>
#include <boost/shared_ptr.hpp>
#include <iostream>
#include <string>
#include <vector>

namespace DAQCONFIG = dbutils::daqconfig;

namespace dbutils {
namespace dcs {

/**
 * Worker class to handle fetching and formatting of APD
 * configuration data.
 *
 * @author Kurt Biery
 * @version $Revision: 1.8 $ $Date: 2013/03/08 20:33:53 $
 */

class APDSettings {

 public:

    static const std::string WARM_COOLING_STATE;
    static const std::string COLD_COOLING_STATE;

    // ********************
    // *** Constructors ***
    // ********************

    APDSettings(const int64_t& globalConfigId,
                const std::vector<std::string>& dcmList);

    APDSettings(DAQCONFIG::GlobalNameSpec nameSpec,
                const std::vector<std::string>& dcmList);

    APDSettings(DAQCONFIG::SubsystemNameSpec nameSpec,
                const std::vector<std::string>& dcmList);

    virtual ~APDSettings() {};

    // *****************
    // *** Accessors ***
    // *****************

    virtual bool hasValidData() const {return _hasValidData;}

    bool getTemperature(const std::string& dcmName, const int& febId,
                        int& temperature);
    bool getCoolingEnableFlag(const std::string& dcmName, const int& febId,
                              bool& isEnabled);
    bool getVoltage(const std::string& dcmName, const int& febId,
                    int& regulatorSetting);

    bool getTemperature(const std::string& dcmName, const int& febId,
                        double& temperature);
    bool getTemperatureEnable(const std::string& dcmName, const int& febId,
                              bool& enableFlag);
    bool getVoltage(const std::string& dcmName, const int& febId,
                    const std::string& coolingState, int& regulatorSetting);

    bool getTemperatures(const std::string& dcmName,
                         std::vector<int>& temperatureList);
    bool getCoolingEnableMask(const std::string& dcmName,
                              int64_t& enableMask);
    bool getVoltages(const std::string& dcmName,
                     std::vector<int>& voltageList);

    bool getTemperatures(const std::string& dcmName,
                         std::vector<double>& temperatureList);
    bool getTemperatureEnableMask(const std::string& dcmName,
                                  int64_t& enableMask);
    bool getVoltages(const std::string& dcmName,
                     const std::string& coolingState,
                     std::vector<int>& voltageList);

    bool getDryAirChannel(const std::string& dcmName, const int& febId,
                          int& channelNumber);
    bool getDryAirChannels(const std::string& dcmName,
                           std::vector<int>& channelList);

    bool getAlarmThreshold(const std::string& dcmName,
                           const int& febId,
                           const std::string& pvName,
                           const std::string& thresholdType,
                           double& threshold);

 protected:

    bool _fetchAllData(const std::vector<int64_t>& subsysConfigIdList);

    std::vector<std::string> _dcmList;
    bool _hasValidData;

    boost::shared_ptr<DAQCONFIG::ConfigDataTree> _temperatureDataSet;
    boost::shared_ptr<DAQCONFIG::ConfigDataTree> _enableMaskDataSet;
    boost::shared_ptr<DAQCONFIG::ConfigDataTree> _hvAdjustDataSet;
    boost::shared_ptr<DAQCONFIG::ConfigDataTree> _dryAirChannelDataSet;
    boost::shared_ptr<DAQCONFIG::ConfigDataTree> _alarmThresholdDataSet;

};

};  // end of namespace dcs
};  // end of namespace dbutils

#endif
