#ifndef _NOVADAQ_CACHE_H
#define _NOVADAQ_CACHE_H

#include <NovaDAQUtilities/CachePolicy.h>
#include <boost/thread/mutex.hpp>

namespace novadaq {

/**
 * A base class to be used when implementing a cache of data.
 * That is, a set of data that is not expected to change often,
 * but which may need to be updated at some point (either through
 * some automatic means or via user or application request).
 *
 * @author Kurt Biery
 * @version $Revision: 1.1 $ $Date: 2009/11/18 14:12:04 $
 */

class Cache {

 public:

    void flush();

    virtual ~Cache() {};

 protected:

    explicit Cache(CachePolicy policy);

    void _reloadIfNeeded();

    virtual void _loadDataIntoCache() = 0;

    virtual void _clearCachedData() = 0;

 protected:

    CachePolicy _policy;
    bool _cacheIsEmpty;
    boost::mutex _cacheMutex;

};

};  // end of namespace novadaq

#endif
