/* 
 * File:   SHMConvention.h
 * Author: denis
 *
 * This class handles the memory segments conventions between DataLogger and Event Dispatcher
 *
 * Created on April 30, 2012, 4:10 PM
 */

#ifndef SHMCONVENTION_H
#define	SHMCONVENTION_H
#include <stdint.h>

namespace novadaq {

    /// Enumeration of the Offline frameworks
    enum OFFLINE_FRAMEWORK{
         ART_FRAMEWORK  = 1
        ,FMWK_FRAMEWORK
        ,LAST_FRAMEWORK
    };

    class SHMConvention {
        public:

            SHMConvention();
            ~SHMConvention();

            /// Get Shared Memory Segment ID, based on Partition and Framework ID
            static uint32_t getSHMid(uint32_t partition, OFFLINE_FRAMEWORK framework);

            /// Get Total number of Shared memory segments used by DataLogger (right now by default it's 2 - ART and FMWK)
            static uint32_t getNumberOfSHMSegments();

            /// Get Base size for the initial shm query (1k)
            static uint32_t getBaseSHMSize();
        
            /// Get size of shared memory segment (32M default)
            static uint32_t getSHMSize();


        private:
            
            static const uint32_t p_letter             = 0x50;      //< ASCII letter 'P'
            static const uint32_t BASE_SHM_SIZE        = 0x400;     //< Base size for the initial shm query (1k)
            static const uint32_t SHM_memsize          = 0x2000000; //< Default size of shared memory segment is 32M
            static const uint32_t DEFAULT_SHM_SEGMENTS = 2;         //< number of Shared memory segments used by DataLogger  (right now by default it's 2 - ART and FMWK)

            /// Masks for the 4 bytes of the unsigned int
            enum IDMASKS{
                MASK_1            = 0x000000FF,
                MASK_2            = 0x0000FF00,
                MASK_3            = 0x00FF0000,
                MASK_4            = 0xFF000000,
            };

            /// Corresponding shifts for the 4 bytes of the unsigned int
            enum IDSHIFTS{
                SHIFT_1            = 00,
                SHIFT_2            = 8,
                SHIFT_3            = 16,
                SHIFT_4            = 24,
            };

    };
}

#endif	/* SHMCONVENTION_H */

