#include "DAQChannelMap/DAQChannelMapOLD.h"
#include "DAQChannelMap/BitFields.h"
#include "DAQChannelMap/boundscheck.h"
#include "NovaDAQConventions/DAQConventions.h"

#include <stdio.h>
#include <iostream>

using namespace daqchannelmap;

/**********************************************************************************
 * DAQChannelMap Class Implimentation                                             *
 **********************************************************************************/

// Initialize the singleton pointer to NULL
DAQChannelMapOLD*  DAQChannelMapOLD::_instanceID = NULL;

//------------------------------------------------------------------------------
/*! getInstance method returns the current instance if there is one
  or creates the initial instance if there is not one */
DAQChannelMapOLD* DAQChannelMapOLD::getInstance(){
  if(_instanceID == NULL){	               // Check if there is an instance active
    _instanceID = new DAQChannelMapOLD();         // If there is not, make a new one
  }
  return _instanceID;
}

//------------------------------------------------------------------------------
DAQChannelMapOLD::DAQChannelMapOLD(){ }// end of DAQChannelMapOLD constructor
DAQChannelMapOLD::~DAQChannelMapOLD(){ }		//!< Default Destructor

//------------------------------------------------------------------------------
dchan DAQChannelMapOLD::encodeDChan(int detector, diblock_t diblock,
				 dcm_id_t dcmid,    dcm_link_t dcmport,
                                 pixel_t pixel) const{
  dchan chan = 0x0;
  BITFIELDSET(chan,detector, MASK_DETECTOR, SHIFT_DETECTOR);
  BITFIELDSET(chan,diblock,  MASK_DIBLOCK,  SHIFT_DIBLOCK);
  BITFIELDSET(chan,dcmid,    MASK_DCMID,    SHIFT_DCMID);
  BITFIELDSET(chan,dcmport,  MASK_DCMPORT,  SHIFT_DCMPORT);
  BITFIELDSET(chan,pixel,    MASK_PIXELID,  SHIFT_PIXELID);
  return chan;
}

//------------------------------------------------------------------------------
lchan DAQChannelMapOLD::encodeLChan(int             detector,
				 offline_plane_t plane_0,
				 offline_cell_t  cell_0) const{
  lchan chan = 0x0;
  BITFIELDSET(chan,detector, MASK_OFFLINE_DETECTOR, SHIFT_OFFLINE_DETECTOR);
  BITFIELDSET(chan,plane_0 , MASK_OFFLINE_PLANE   , SHIFT_OFFLINE_PLANE);
  BITFIELDSET(chan,cell_0  , MASK_OFFLINE_CELL    , SHIFT_OFFLINE_CELL);
  return chan;
}

//------------------------------------------------------------------------------
dchan DAQChannelMapOLD::encodeDChan(lchan chan) {

    const int             detector = getDetectorOffline(chan);
    const offline_plane_t plane_0  = getPlaneOffline(chan);
    const offline_cell_t  cell_0   = getCellOffline(chan);
    //const DiBlock_TYPE diblock_type = getDiBlockTypeOffline(chan);
    //uint32_t cell_1 = 1;

//     // Test for MC DCM 2...
//     if(detector==novadaq::cnv::kNEARDET && diblock_type==MC_ND_DIBLOCK){
//       const DetPlane_TYPE plane_type = getPlaneTypeOffline(chan);
//       const dcm_id_t dcm_id = getDCM(detector, plane_type, diblock_type, 10000203);

//       if(dcm_id==2){
// 	cell_1 = 63 - cell_0 - 32;
//       }
//       else cell_1 = cell_0;
//     }
//     else cell_1 = cell_0;
	  

    return encodeDChan(detector, plane_0, cell_0);
}


//------------------------------------------------------------------------------
dchan DAQChannelMapOLD::encodeDChan( const int             detector
                                 ,const offline_plane_t plane_0
                                 ,const offline_cell_t  cell_0   ) const {

  const block_t         block_number   = getBlockNumber(detector, plane_0);
  const diblock_t       diblock_number = getDiblockNumber(block_number);
  const DiBlock_TYPE    diblock_type   = getDiBlockType(detector, diblock_number);
  const DetPlane_TYPE   plane_type     = getPlaneType(detector, plane_0);
  const online_cell_t   online_cell_0  = offlineCellToOnlineCell(cell_0, detector, plane_type, diblock_type);


  const dcm_id_t        dcm_id   = getDCM(detector, plane_type, diblock_type, online_cell_0);
  const dcm_link_t      dcm_link = getDCMLink(detector, plane_0, online_cell_0, diblock_type, block_number, plane_type);

  const pixel_t         pixel    = getPixel(online_cell_0, diblock_type);
  
  const apd_pixel_t     apd_pixel = pixel2APDPixel(pixel);

  
  //std::cout<<" diblock:  "<<diblock_number<<"  diblock type:  "<<diblock_type<<" dcm:  "<<dcm_id<<"  feb:  "<<dcm_link<<"  pixel:  "<<apd_pixel<<std::endl;

  return encodeDChan(detector, diblock_number, dcm_id, dcm_link, apd_pixel);
    
}


//------------------------------------------------------------------------------
lchan DAQChannelMapOLD::encodeLChan(dchan chan) {

    const int             det                         = getDetector(chan);
    const diblock_t       diblock_id                  = getDiBlock(chan);
    const dcm_id_t        dcm_id                      = getDCM(chan);
    const dcm_link_t      dcm_link_id                 = getDCMLink(chan);
    const apd_pixel_t     apd_pixel_id                = getPixel(chan);


    const offline_plane_t plane_0 = getPlane(det, diblock_id, dcm_id, dcm_link_id);
    const offline_cell_t  cell_0  = getCell(det, diblock_id, dcm_id, dcm_link_id, apd_pixel_id);

    return encodeLChan(det, plane_0, cell_0);
}

//------------------------------------------------------------------------------
int DAQChannelMapOLD::getDetector(dchan daqchan) const{
  return (int)BOUNDSCHECK(
		     BITFIELDGET(daqchan,MASK_DETECTOR,SHIFT_DETECTOR),
		     novadaq::cnv::kUNKNOWN_DET+1,
		     novadaq::cnv::kNDetector-1, ERROR_UNKNOWN_DET);
}

//------------------------------------------------------------------------------
diblock_t DAQChannelMapOLD::getDiBlock(dchan daqchan) const{
  return BOUNDSCHECK(
		     BITFIELDGET(daqchan,MASK_DIBLOCK,SHIFT_DIBLOCK),
		     MIN_DIBLOCK, MAX_DIBLOCK, ERROR_DIBLOCK_OUTRANGE);
}

//------------------------------------------------------------------------------
dcm_id_t DAQChannelMapOLD::getDCM(dchan daqchan) const{
  return  BOUNDSCHECK(
		     BITFIELDGET(daqchan,MASK_DCMID,SHIFT_DCMID),
		     MIN_DCM,MAX_DCM, ERROR_DCM_OUTRANGE);
}

//------------------------------------------------------------------------------
dcm_link_t DAQChannelMapOLD::getDCMLink(dchan daqchan) const {
  return BOUNDSCHECK(
		     BITFIELDGET(daqchan,MASK_DCMPORT,SHIFT_DCMPORT),
		     MIN_PORT,MAX_PORT, ERROR_PORT_OUTRANGE);
}

//------------------------------------------------------------------------------
pixel_t DAQChannelMapOLD::getPixel(dchan daqchan) const {
  return BOUNDSCHECK(
		     BITFIELDGET(daqchan,MASK_PIXELID,SHIFT_PIXELID),
		     MIN_PIXEL,MAX_PIXEL, ERROR_PIXEL_OUTRANGE);
}

//------------------------------------------------------------------------------
DetView_TYPE DAQChannelMapOLD::getView(dchan daqchan) const {
  const int detector_id  = getDetector(daqchan);
  const dcm_id_t      dcm_id       = getDCM(daqchan);
  const DiBlock_TYPE  diblock_type = getDiBlockType(daqchan);

  const DetPlane_TYPE plane_type   = getPlaneType(detector_id, dcm_id, diblock_type);

  return detPlaneToDetView(plane_type);
}

//------------------------------------------------------------------------------
//! Returns the type of diblock that the channel was on
DiBlock_TYPE DAQChannelMapOLD::getDiBlockType(dchan daqchan) const {
    const int       detector       = getDetector(daqchan);
    const diblock_t diblock_number = getDiBlock(daqchan);

    return getDiBlockType(detector, diblock_number);//calling base
}

//------------------------------------------------------------------------------
//! Returns the Plane in coordinates relative to the dcm (i.e. 0-31)
offline_plane_t DAQChannelMapOLD::getPlane(dchan daqchan) {

    const int        detector_id = getDetector(daqchan);
    const diblock_t  diblock_id  = getDiBlock(daqchan);
    const dcm_id_t   dcm_id      = getDCM(daqchan);
    const dcm_link_t dcm_link_id = getDCMLink(daqchan);

    return getPlane(detector_id, diblock_id, dcm_id, dcm_link_id);
}

//------------------------------------------------------------------------------
//! Returns the Plane in coordinates relative to the dcm (i.e. 0-31)
offline_plane_t DAQChannelMapOLD::getPlane(const int detector_id, const diblock_t diblock_id, const dcm_id_t dcm_id, const dcm_link_t dcm_link_id) {

    /// Now Based on (detector_id, diblock_id, dcm_id, dcm_link_id), calculate these variables
    const DiBlock_TYPE    diblock_type                = getDiBlockType(detector_id, diblock_id);// calling base
    const offline_plane_t local_plane_in_diblock_view = getLocalPlaneInDiblockView(detector_id, dcm_link_id, diblock_type);
    const DetPlane_TYPE   plane_type                  = getPlaneType(detector_id, dcm_id, diblock_type);

    return  getNumberOfPlanesBeforeDiblock(detector_id, diblock_id)
           +getLocalPlaneInDiblock(local_plane_in_diblock_view, diblock_type, plane_type);
}

//------------------------------------------------------------------------------
// Offline Channel Decoding
int DAQChannelMapOLD::getDetectorOffline(lchan chan) const {

  return BITFIELDGET(chan,MASK_OFFLINE_DETECTOR,SHIFT_OFFLINE_DETECTOR);


  return (int)BOUNDSCHECK(
		     BITFIELDGET(chan,MASK_OFFLINE_DETECTOR,SHIFT_OFFLINE_DETECTOR),
		     novadaq::cnv::kUNKNOWN_DET+1,
		     novadaq::cnv::kNDetector-1, ERROR_UNKNOWN_DET);
}

//------------------------------------------------------------------------------
offline_plane_t DAQChannelMapOLD::getPlaneOffline(lchan chan) const {

    return BITFIELDGET(chan, MASK_OFFLINE_PLANE, SHIFT_OFFLINE_PLANE);

    return BOUNDSCHECK(
		     BITFIELDGET(chan, MASK_OFFLINE_PLANE, SHIFT_OFFLINE_PLANE),
		     0,NUM_PLANES_DIBLOCK[getDetectorOffline(chan)]*NUM_DIBLOCK_DET[getDetectorOffline(chan)], ERROR_PLANE_OUTRANGE);
}

//------------------------------------------------------------------------------
offline_cell_t DAQChannelMapOLD::getCellOffline(lchan chan) const {
  return BOUNDSCHECK(
		     BITFIELDGET(chan,MASK_OFFLINE_CELL,SHIFT_OFFLINE_CELL),
	 	     0,NUM_CELLS*NUM_PLANES_DIBLOCK[getDetectorOffline(chan)]*NUM_DIBLOCK_DET[getDetectorOffline(chan)], ERROR_CELL_OUTRANGE);
}

//------------------------------------------------------------------------------
uint32_t DAQChannelMapOLD::getBlockOffline(lchan chan) const {

  const int             detector = getDetectorOffline(chan);
  const offline_plane_t plane_0  = getPlaneOffline(chan); // plane starts from 0 in Offline

  return getBlockNumber(detector, plane_0);
}
//------------------------------------------------------------------------------
uint32_t DAQChannelMapOLD::getDiBlockOffline(lchan chan) const {
  const int             detector = getDetectorOffline(chan);
  const offline_plane_t plane_0  = getPlaneOffline(chan); // plane starts from 0 in Offline

  return getDiblockNumber(detector, plane_0);
}

//------------------------------------------------------------------------------
//  Returns the module relative to the current dcm that a channel belongs to
//  (normally 0 or 1 but could be 2 for the NDOS or NearDet muon catchers.)
//
//  For FarDet and NearDet:
//    Local Module 0 refers to DCM ports 0-31
//    Local Module 1 refers to DCM ports 32-63
//  For NDOS:
//    Local Module 0 refers to DCM ports in range 0-15, 32-47
//    Local Module 1 refers to DCM ports in range 16-31, 48-63 
//------------------------------------------------------------------------------
uint32_t DAQChannelMapOLD::getLocalModuleInDCM(const int detector_id, dcm_id_t dcm_id, dcm_link_t dcm_link, const DiBlock_TYPE diblock_type) const {

  // if(detector_id == novadaq::cnv::kFARDET) return dcm_link/32;
  
  /// Compute for FarDet or NearDet according to the above comment
  if(detector_id == novadaq::cnv::kNEARDET){
    
    /// NearDet muon catcher must be treated differently
    if(detector_id == novadaq::cnv::kNEARDET && diblock_type == MC_ND_DIBLOCK) {

      // The mapping for each DCM in the NearDet MC is different (go team!!!)
      // There are 11 planes and the FEBs are plugged in in numerical order with
      // no gaps (0-32 for the top, 0-21 for the side) so dividing by 11 tells you
      // which module you are in.
      if(dcm_id == 1) {
	if(dcm_link > 32) return -1;  // Flag as uninstrumented.
	else              return dcm_link/11;
      }

      // The FEBs for DCM 2 are plugged in in reverse order, which is why I have
      // to do the 1-(module) thing.
      if(dcm_id == 2) {
	if(dcm_link > 21) return -1;  // Flag as uninstrumented.
	else return dcm_link/11;
	// else              return 1 - (dcm_link/11);  This was implemented when the FEBs were switched for Mod 0 and Mod 1.
      }

      // Otherwise this is DCM 3 or 4 which for the NearDet MC is uninstrumented.
      return -1;

    }
    
    // Otherwise this is a "normal" NearDet diblock.
    if(dcm_id==3){
      if(dcm_link/32 == 0) return 1;
      else if(dcm_link/32 == 1) return 0;
    }
    else return dcm_link / 32 ;
  }

  /// Otherwise this must be NDOS, and compute according to the above comment
  
  if(detector_id == novadaq::cnv::kNDOS){
    /// Muon catcher screws things up
    if(diblock_type == MC_DIBLOCK) return dcm_link / 16;

    return (dcm_link % 32)/16;
  }

  /// If not NearDet or NDOS, then must be FarDet.
  return dcm_link/32;
}

//------------------------------------------------------------------------------
//! Returns the Cell in coordinates relative to the detector view (i.e. 1-"a big number" for far det)
//  Cells are referenced in the offline 0-95 (NDOS) with cell 0 at the snout end in X-View
//  And cell 0 at the bottom of the detector
offline_cell_t DAQChannelMapOLD::getCell(dchan daqchan) const {

    /// straight extractions
    const int           detector_id    = getDetector(daqchan);
    const diblock_t     diblock_number = getDiBlock(daqchan);
    const dcm_id_t      dcm_id         = getDCM(daqchan);
    const dcm_link_t    dcm_link_id    = getDCMLink(daqchan);
    const apd_pixel_t   apd_pixel_id   = getPixel(daqchan);

    return getCell(detector_id, diblock_number, dcm_id, dcm_link_id, apd_pixel_id);
}


//------------------------------------------------------------------------------
offline_cell_t DAQChannelMapOLD::getCell(const int     detector_id,
                                const diblock_t     diblock_number,
                                const dcm_id_t      dcm_id        ,
                                const dcm_link_t    dcm_link_id   ,
                                const apd_pixel_t   apd_pixel_id  
                               ) const {


    const DiBlock_TYPE  diblock_type   = getDiBlockType(detector_id, diblock_number);

    const pixel_t       pixel_id       = APDPixel2Pixel(apd_pixel_id);

    uint32_t number_of_x_dcm = 0;
    const DetPlane_TYPE plane_type     = getPlaneType(detector_id, dcm_id, diblock_type, number_of_x_dcm);
    const online_cell_t online_cell    = getOnlineCell(detector_id, dcm_id, dcm_link_id, pixel_id, diblock_type, plane_type, number_of_x_dcm);

    return onlineCellToOfflineCell(online_cell, detector_id, plane_type, diblock_type);
}



//------------------------------------------------------------------------------
DiBlock_TYPE DAQChannelMapOLD::getDiBlockTypeOffline(lchan chan) const {
  const diblock_t diblock_id = getDiBlockOffline(chan);
  const int       detector   = getDetectorOffline(chan);

  return getDiBlockType(detector, diblock_id);// calling base
}

//------------------------------------------------------------------------------
DetBlock_TYPE DAQChannelMapOLD::getBlockType(lchan logicalchan) const {
    const int           detector      = getDetectorOffline(logicalchan);
    const uint32_t      block_number  = getBlockOffline(logicalchan);

    return getBlockType(block_number, detector);
}

//------------------------------------------------------------------------------
DetBlock_TYPE DAQChannelMapOLD::getBlockType(block_t blockID, int detector) const  {

    const diblock_t     diblockID                     = getDiblockNumber(blockID); //< we count diblocks from 1
    const uint32_t      local_block_number_in_diblock = getLocalBlockInDiblock(blockID);       //< we count from 0

    const DiBlock_TYPE  diblock_type                  = getDiBlockType(detector, diblockID);// calling base
    
    return getBlockType(diblock_type, local_block_number_in_diblock);
}

//------------------------------------------------------------------------------
// Returns local plane number for the diblock from offline channel
offline_plane_t DAQChannelMapOLD::getLocalPlaneInDiblockOffline(lchan chan) const {
  const int             detector = getDetectorOffline(chan);
  const offline_plane_t plane_0  = getPlaneOffline(chan);    //< plane starts from 0 in Offline

  return getLocalPlaneInDiblock(detector, plane_0);
}

//------------------------------------------------------------------------------
offline_plane_t DAQChannelMapOLD::getLocalPlaneInBlockOffline(lchan chan) const {
  const int             detector = getDetectorOffline(chan);
  const offline_plane_t plane_0  = getPlaneOffline(chan); // plane starts from 0 in Offline

  return getLocalPlaneInBlock(detector, plane_0);
}

//------------------------------------------------------------------------------
DetView_TYPE DAQChannelMapOLD::getViewOffline(lchan chan) const {
    return detPlaneToDetView(getPlaneTypeOffline(chan));
}

//------------------------------------------------------------------------------
DetPlane_TYPE DAQChannelMapOLD::getPlaneTypeOffline(lchan chan) const {
  const DetBlock_TYPE block_type           = getBlockType(chan);
  const uint32_t      local_plane_in_block = getLocalPlaneInBlockOffline(chan);

  const DetPlane_TYPE start_plane_type     = getStartPlaneTypeInBlock(block_type);

  /// If local plane is even, then the view is the same as the one that the block starts with
  if(EVEN(local_plane_in_block)) return start_plane_type;

  return getOtherPlaneType(start_plane_type);
}

//------------------------------------------------------------------------------
offline_plane_t DAQChannelMapOLD::getLocalPlaneInDiblockViewOffline(lchan chan) const {

    const offline_plane_t local_plane_in_block = getLocalPlaneInBlockOffline(chan);

    offline_plane_t       local_plane_in_block_view  = local_plane_in_block / 2;

    const int local_block_in_diblock = getLocalBlockInDiblockOffline(chan);

    if(local_block_in_diblock == 0) {
        return local_plane_in_block_view;
    }

    /// otherwise, we need to calculate how many planes of the same view
    /// were in the first diblock

    const int           detector                                 = getDetectorOffline(chan);

    /// The first block of the diblock - it's the one before the current
    const block_t       block_id_of_the_first_block_in_diblock   = getBlockOffline(chan) - 1;
    const DetBlock_TYPE block_type_of_the_first_block_in_diblock = getBlockType(block_id_of_the_first_block_in_diblock, detector);

    /// View of the plane
    const DetPlane_TYPE plane_type = getPlaneTypeOffline(chan);

    /// Add the number of planes in the view for that diblock
    local_plane_in_block_view += getNumberOfPlaneTypeInBlock(block_type_of_the_first_block_in_diblock, plane_type);

    return local_plane_in_block_view;
}

//------------------------------------------------------------------------------
offline_plane_t DAQChannelMapOLD::getLocalPlaneInDiblockView( const int             detector
                                                          ,const offline_plane_t plane_0
                                                         ) const {
    const block_t block_number = getBlockNumber(detector, plane_0);
    return getLocalPlaneInDiblockView(detector, plane_0, block_number);
}

//------------------------------------------------------------------------------
offline_plane_t DAQChannelMapOLD::getLocalPlaneInDiblockView( const int             detector
                                                          ,const offline_plane_t plane_0
                                                          ,const block_t         block_number
                                                         ) const {

    const offline_plane_t local_plane_in_block         = getLocalPlaneInBlock(detector, plane_0);

    offline_plane_t       local_plane_in_diblock_view  = local_plane_in_block / 2;

    const int             local_block_in_diblock       = getLocalBlockInDiblock(block_number);

    if(local_block_in_diblock == 0){
        return local_plane_in_diblock_view;
    }

    /// otherwise, we need to calculate how many planes of the same view
    /// were in the first diblock

    /// The first block of the diblock - it's the one before the current
    const block_t       block_id_of_the_first_block_in_diblock   = block_number - 1;
    const DetBlock_TYPE block_type_of_the_first_block_in_diblock = getBlockType(block_id_of_the_first_block_in_diblock, detector);

    /// View of the plane
    const DetPlane_TYPE plane_type = getPlaneType(detector, plane_0);

    /// Add the number of planes in the view for that diblock
    local_plane_in_diblock_view += getNumberOfPlaneTypeInBlock(block_type_of_the_first_block_in_diblock, plane_type);

    return local_plane_in_diblock_view;
}


//------------------------------------------------------------------------------
offline_plane_t DAQChannelMapOLD::getLocalPlaneInDiblockView( const int             detector
                                                          ,const offline_plane_t plane_0
                                                          ,const block_t         block_number
                                                          ,const DetPlane_TYPE   plane_type
                                                         ) const {

    const offline_plane_t local_plane_in_block         = getLocalPlaneInBlock(detector, plane_0);

    offline_plane_t       local_plane_in_diblock_view  = local_plane_in_block / 2;

    const int             local_block_in_diblock       = getLocalBlockInDiblock(block_number);

    if(local_block_in_diblock == 0){
        return local_plane_in_diblock_view;
    }

    /// otherwise, we need to calculate how many planes of the same view
    /// were in the first diblock

    /// The first block of the diblock - it's the one before the current
    const block_t       block_id_of_the_first_block_in_diblock   = block_number - 1;
    const DetBlock_TYPE block_type_of_the_first_block_in_diblock = getBlockType(block_id_of_the_first_block_in_diblock, detector);

    /// Add the number of planes in the view for that diblock
    local_plane_in_diblock_view += getNumberOfPlaneTypeInBlock(block_type_of_the_first_block_in_diblock, plane_type);

    return local_plane_in_diblock_view;
}


//------------------------------------------------------------------------------
// Returns the plane number of the channel in the X or Y view
offline_plane_t DAQChannelMapOLD::getPlaneViewOffline(lchan chan){
  
  const diblock_t     diblock    = getDiBlockOffline(chan);
  const int           detector   = getDetectorOffline(chan);
  const DetPlane_TYPE plane_type = getPlaneTypeOffline(chan);

  // Sum the number of planes in this view prior to the current diblock
  // that the channel is from

  /// the plane number that the current channel is on in the local diblock
  uint32_t total_planes = getLocalPlaneInDiblockViewOffline(chan);

  for(uint32_t idiblock = 0; idiblock < diblock; ++idiblock){
    const DiBlock_TYPE diblock_type = getDiBlockType(detector, idiblock);// calling base
    total_planes +=  getNumberOfPlaneTypeInDiblock(diblock_type, plane_type);
  }// end of loop over diblocks

  return total_planes;
}

//------------------------------------------------------------------------------
dcm_id_t DAQChannelMapOLD::getDCMOffline(lchan chan) const {

    const int             detector = getDetectorOffline(chan);
    const offline_plane_t plane_0  = getPlaneOffline(chan);
    const offline_cell_t  cell_0   = getCellOffline(chan);

    return getDCM(detector, plane_0, cell_0);
}


//------------------------------------------------------------------------------
dcm_id_t DAQChannelMapOLD::getDCM( const int             detector
                               ,const offline_plane_t plane_offline_0
                               ,const offline_cell_t  cell_offline_0) const{

  const diblock_t     diblock_number    = getDiblockNumber(detector, plane_offline_0);
  const DiBlock_TYPE  diblock_type      = getDiBlockType(detector, diblock_number);

  const DetPlane_TYPE plane_type        = getPlaneType(detector, plane_offline_0);

  const online_cell_t online_cell_0     = offlineCellToOnlineCell(cell_offline_0, detector, plane_type, diblock_type);

  return getDCM(detector, plane_type, diblock_type, online_cell_0);
}


//------------------------------------------------------------------------------
dcm_id_t DAQChannelMapOLD::getDCM( const int           detector
                               ,const DetPlane_TYPE plane_type
                               ,const DiBlock_TYPE  diblock_type
                               ,const online_cell_t online_cell_0) const
{

  const dcm_id_t first_dcm_in_view = getFirstDCMInView(detector, plane_type, diblock_type);

  // Treat the NearDet muon catcher as a special case.
  if(detector == novadaq::cnv::kNEARDET && diblock_type == MC_ND_DIBLOCK) {
    // This one is simple:  If this is a vertical (XZ) plane, then this is DCM 1.
    // If this is a horizontal (YZ) plane, then this is DCM 2.
    if(plane_type == XZ_PLANE) return 1;
    else                       return 2;
  }

  /// For NearDet the first DCM is half-populated
  //  NOTE: "first_dcm_in_view == 1" really is asking "Is this the XZ_VIEW?" but this is
  //        how it was originally coded and I don't want to mess with something that
  //        isn't broken.
  if(detector == novadaq::cnv::kNEARDET && first_dcm_in_view == 1) {
      if(online_cell_0 >= NUM_CELLS) 
          return first_dcm_in_view + 1;

      return first_dcm_in_view;
  }

  return first_dcm_in_view + online_cell_0 / getNumberOfCellsPerDCM(diblock_type);
}


//------------------------------------------------------------------------------
pixel_t  DAQChannelMapOLD::getPixel( const int             detector
                                 ,const offline_plane_t plane_0
                                 ,const offline_cell_t  cell_0  ) const {

    const DetPlane_TYPE plane_type        = getPlaneType(detector, plane_0);
    const diblock_t     diblock_number    = getDiblockNumber(detector, plane_0);
    const DiBlock_TYPE  diblock_type      = getDiBlockType(detector, diblock_number);
    const online_cell_t online_cell_0     = offlineCellToOnlineCell(cell_0, detector, plane_type, diblock_type);

    return getPixel(online_cell_0, diblock_type);
}

//------------------------------------------------------------------------------
pixel_t  DAQChannelMapOLD::getPixel( const online_cell_t online_cell_0
                                 ,const DiBlock_TYPE  diblock_type) const {
    const online_cell_t local_cell_in_dcm = getLocalCellInDCM(online_cell_0, diblock_type);

    return local_cell_in_dcm % NUM_PIXELS;
  }


//------------------------------------------------------------------------------
dcm_link_t DAQChannelMapOLD::getDCMLinkOffline(lchan chan) const {

    const int             detector = getDetectorOffline(chan);
    const offline_plane_t plane_0  = getPlaneOffline(chan);
    const offline_cell_t  cell_0   = getCellOffline(chan);

    return getDCMLink(detector, plane_0, cell_0);
}

//------------------------------------------------------------------------------
dcm_link_t DAQChannelMapOLD::getDCMLink( const int             detector
                                     ,const offline_plane_t plane_0
                                     ,const offline_cell_t  cell_0) const {


  const DetPlane_TYPE   plane_type                  = getPlaneType(detector, plane_0);

  const block_t         block_number                = getBlockNumber(detector, plane_0);
  const diblock_t       diblock_number              = getDiblockNumber(block_number);
  const DiBlock_TYPE    diblock_type                = getDiBlockType(detector, diblock_number);

  const online_cell_t   online_cell_0               = offlineCellToOnlineCell(cell_0, detector, plane_type, diblock_type);


  return getDCMLink(detector, plane_0, online_cell_0, diblock_type, block_number, plane_type);
}// end of getDCMLinkOffline


//------------------------------------------------------------------------------
dcm_link_t DAQChannelMapOLD::getDCMLink( const int             detector
                                     ,const offline_plane_t plane_0
                                     ,const online_cell_t   online_cell_0
                                     ,const DiBlock_TYPE    diblock_type
                                     ,const block_t         block_number
                                     ,const DetPlane_TYPE   plane_type) const {

  // For the NearDet, FEBs 32-63 in DCM 1 are not instrumented but there is no gap in cells.
  // So if we are in DCM 2, we need to shift global cell number by 32 to compensate for this gap.
  const uint32_t dcm       = getDCM(detector, plane_type, diblock_type, online_cell_0);
  online_cell_t  temp_cell = online_cell_0;
  if(detector == novadaq::cnv::kNEARDET && dcm == 2 && diblock_type != MC_ND_DIBLOCK) 
    temp_cell += 32;

  const uint32_t        local_module                = getLocalModuleInDCMOffline(temp_cell, diblock_type); //< either 0 or 1 (or 2 in the MC)
  const offline_plane_t local_plane_in_diblock_view = getLocalPlaneInDiblockView(detector, plane_0, block_number, plane_type);

  /// For FarDet and NearDet, the calculation is different than it is for NDOS
  if(detector == novadaq::cnv::kFARDET || detector == novadaq::cnv::kNEARDET){

    // Treat the NearDet muon catcher as a special case
    if(detector == novadaq::cnv::kNEARDET && diblock_type == MC_ND_DIBLOCK) {
      return local_plane_in_diblock_view + 11*local_module;
      // comment..
      // return local_plane_in_diblock_view + 11*(1-local_module);
    }

    // Otherwise this is a "normal" FarDet or NearDet diblock.
    return local_plane_in_diblock_view + 32*local_module;
  }

  // Otherwise, this must be NDOS

  /// Calculate DCM link number according to NOvA-doc-4219
  /// This math is true for NDOS only
  dcm_link_t dcm_link = local_plane_in_diblock_view + 16*local_module;
  if(local_plane_in_diblock_view > 15) dcm_link += 16;

  return dcm_link;
}// end of getDCMLinkOffline



//------------------------------------------------------------------------------
// If it's the upstream block in diblock, then it's 0, if not, it's 1
uint32_t DAQChannelMapOLD::getLocalBlockInDiblockOffline(lchan logicalchan) const {
    const uint32_t block = getBlockOffline(logicalchan);

    return getLocalBlockInDiblock(block);
}// end of getLocalBlockInDiblockOffline

//------------------------------------------------------------------------------
dchan DAQChannelMapOLD::l2d(offline_plane_t plane, offline_cell_t cell, int detector)  {return encodeDChan(encodeLChan(detector,plane,cell));};

//------------------------------------------------------------------------------
/// First DCM in the view. if the channel is in a XZ plane, the DCM is 1. Otherwise, it's 7 for the far detector,
/// 3 for the near detector, and 2 for NDOS
dcm_id_t DAQChannelMapOLD::getFirstDCMInViewOffline(lchan chan) const {

    const DetPlane_TYPE plane_type   = getPlaneTypeOffline(chan);
    const int           detector     = getDetectorOffline(chan);
    const DiBlock_TYPE  diblock_type = getDiBlockTypeOffline(chan);

    return getFirstDCMInView(detector, plane_type, diblock_type);
}

//------------------------------------------------------------------------------
/// First DCM in the view. if the channel is in a XZ plane, the DCM is 1. Otherwise, it's 7 for the far detector,
/// 3 for the near detector, and 2 for NDOS
dcm_id_t DAQChannelMapOLD::getFirstDCMInView(int detector, DetPlane_TYPE plane_type, DiBlock_TYPE diblock_type) const {

    /// If it's XZ plane, then the first DCM is 1
    if(plane_type == XZ_PLANE) return 1;

    /// Otherwise, it's the next DCM after the last XZ DCM
    return getNumberOfXDCM(detector, diblock_type) + 1;
}

//------------------------------------------------------------------------------
dcm_id_t DAQChannelMapOLD::getFirstDCMInView(const DetPlane_TYPE plane_type, const uint32_t number_of_x_dcms) const{

  // WARNING:  This functions allows you to override the constants defined in DAQChannelMapOLDConstants.h!!!

    /// If it's XZ plane, then the first DCM is 1
    if(plane_type == XZ_PLANE) return 1;

    /// Otherwise, it's the next DCM after the last XZ DCM
    return number_of_x_dcms + 1;
}




//------------------------------------------------------------------------------
dcm_id_t  DAQChannelMapOLD::getLastDCMInViewOffline(lchan chan) const {

    const DetPlane_TYPE plane_type   = getPlaneTypeOffline(chan);
    const int           detector     = getDetectorOffline(chan);
    const DiBlock_TYPE  diblock_type = getDiBlockTypeOffline(chan);

    return getLastDCMInView(detector, plane_type, diblock_type);
}
//------------------------------------------------------------------------------
dcm_id_t  DAQChannelMapOLD::getLastDCMInView(int detector, DetPlane_TYPE plane_type, DiBlock_TYPE diblock_type) const {


    /// If it's XZ plane, then the first DCM is NUM_X_DCMS
    if(plane_type == XZ_PLANE) return getNumberOfXDCM(detector, diblock_type);

    /// So, it's vertical (YZ) plane

    return getFirstDCMInView(detector, plane_type, diblock_type) + getNumberOfYDCM(detector, diblock_type) - 1;
}

//------------------------------------------------------------------------------
pixel_t  DAQChannelMapOLD::getPixelOffline(lchan chan) const {

    const int             detector = getDetectorOffline(chan);
    const offline_plane_t plane_0  = getPlaneOffline(chan);
    const offline_cell_t  cell_0   = getCellOffline(chan);

    return getPixel(detector, plane_0, cell_0);

    // NOTE: The 4 lines below were originally in this code un-commented.  I saw no way that
    //       these lines could have been reached due to the return statement above, but Denis
    //       is full of mystery so I didn't want to delete them just in case we needed them...

    // const online_cell_t online_cell_0     = getOnlineCellOffline(chan);
    // const DiBlock_TYPE  diblock_type      = getDiBlockTypeOffline(chan);
    // const online_cell_t local_cell_in_dcm = getLocalCellInDCM(online_cell_0, diblock_type);
    // return local_cell_in_dcm % NUM_PIXELS;
}


//------------------------------------------------------------------------------
uint32_t DAQChannelMapOLD::getLocalModuleInDCMOffline(lchan chan) const {

    const online_cell_t online_cell_0 = getOnlineCellOffline(chan);
    const DiBlock_TYPE  diblock_type  = getDiBlockTypeOffline(chan);

    return getLocalModuleInDCMOffline(online_cell_0, diblock_type);
}

//------------------------------------------------------------------------------
uint32_t DAQChannelMapOLD::getLocalModuleInDCMOffline(online_cell_t online_cell_0, DiBlock_TYPE diblock_type) const{
  
  // NOTE: No special exception was ever made here for the NDOS muon catcher in the original code
  //       despite the comment below.  I don't know if this case is being treated correctly or not.

  /// This is what it should be. But Muon catcher screws things up
  const online_cell_t local_cell_in_dcm = getLocalCellInDCM(online_cell_0, diblock_type);
  
  return local_cell_in_dcm / NUM_PIXELS;
}

//------------------------------------------------------------------------------
online_cell_t DAQChannelMapOLD::getLocalCellInDCM(online_cell_t online_cell_0, DiBlock_TYPE diblock_type)const{

  const online_cell_t number_of_cells_per_dcm = getNumberOfCellsPerDCM(diblock_type);

  // Treat the NearDet muon catcher as a special case since the number of modules in the NearDet MC is not
  // the same as the "normal" NearDet diblocks (and there are no DAQChannelMapOLD constants that specify the
  // difference between number of modules in a NearDet diblock vs. number of modules in a NearDet MC diblock.)
  // It would probably be more work than it is worth to add this, so we will just catch this special case.
  // Since there is only one DCM per view, then online_cell_0 IS the local cell in the DCM in this case (for
  // either view.)
  if(diblock_type == MC_ND_DIBLOCK) return online_cell_0;
  
  // return online_cell_0 - (online_cell_0 / number_of_cells_per_dcm)*number_of_cells_per_dcm;
  // OR...   ...we could just use modulo instead...
  return online_cell_0 % number_of_cells_per_dcm;
}

//------------------------------------------------------------------------------
DetPlane_TYPE DAQChannelMapOLD::getPlaneType(int detector_id, dcm_id_t dcm_id, DiBlock_TYPE diblock_type) const {
        if(dcm_id <= getNumberOfXDCM(detector_id, diblock_type)) return XZ_PLANE;

        return YZ_PLANE;
}

//------------------------------------------------------------------------------
DetPlane_TYPE DAQChannelMapOLD::getPlaneType(int detector_id, dcm_id_t dcm_id, DiBlock_TYPE diblock_type, uint32_t& number_of_x_dcm) const{

  // I don't understand why we are passing in number_of_x_dcm by reference when there is a function that
  // will get this value for you.

  number_of_x_dcm = getNumberOfXDCM(detector_id, diblock_type);
  
  if(dcm_id <= getNumberOfXDCM(detector_id, diblock_type)) return XZ_PLANE;
  return YZ_PLANE;
}

//------------------------------------------------------------------------------
offline_plane_t DAQChannelMapOLD::getLocalPlaneInDiblockView(const int detector, dcm_link_t dcm_link_id, const DiBlock_TYPE diblock_type) const {

  /// For the FarDet and NearDet, the math is simple
  if(detector == novadaq::cnv::kFARDET || detector == novadaq::cnv::kNEARDET){

    // Treat the NearDet muon catcher as a special case.
    if(diblock_type == MC_ND_DIBLOCK) {
      // The FEBs (dcm_links) on the NearDet MC are plugged in in numerical order with no gaps
      // and there are 11 planes in either view.  So dcm_link_id % 11 will tell you the local
      // plane in view for an FEB in this case.
      return dcm_link_id % 11;
    }

    // Otherwise its a normal FarDet or NearDet diblock.
    return dcm_link_id % 32;
  }

  /// Otherwise, this is NDOS.

  /// DCM links are counted from 0. Refer to NOvA-doc-4219.
  if(dcm_link_id < 32) return dcm_link_id % 16;
  
  /// Muon catcher screws things up. We use 1 DCM for muon catcher in each view
  if(diblock_type == MC_DIBLOCK) return (dcm_link_id - 32) % 16;
  
  return 16 + (dcm_link_id - 32) % 16;
  
}

//------------------------------------------------------------------------------
uint32_t DAQChannelMapOLD::getLocalBlockInDiblock(DiBlock_TYPE    diblock_type,
                                               offline_plane_t local_plane_in_diblock_in_view,
                                               DetPlane_TYPE   plane_type) const {

  // WARNING:  Hardcoded assumption of 2 blocks per diblock!

    const DetBlock_TYPE first_block_of_diblock_type = getBlockType(diblock_type, (uint32_t)0);/// calling base

    /// Get Number of planes of the same type in that first diblock.
    /// Needed to figure out which block current plane belongs to
    const uint32_t number_of_planes_of_the_same_type_in_first_block_of_the_diblock = getNumberOfPlaneTypeInBlock(first_block_of_diblock_type, plane_type);

    if(local_plane_in_diblock_in_view < number_of_planes_of_the_same_type_in_first_block_of_the_diblock) return 0;

    return 1;
}

//------------------------------------------------------------------------------
uint32_t DAQChannelMapOLD::getLocalBlockInDiblock(const int detector, const offline_plane_t plane_0) const{
    const uint32_t block_number = getBlockNumber(detector, plane_0);
    return getLocalBlockInDiblock(block_number);
}


//------------------------------------------------------------------------------
/// Optionally we can also calculate DetBlock_TYPE
uint32_t DAQChannelMapOLD::getLocalBlockInDiblock(const DiBlock_TYPE    diblock_type,
                                               const offline_plane_t local_plane_in_diblock_in_view,
                                               const DetPlane_TYPE   plane_type,
                                               DetBlock_TYPE&        output_block_type,
                                               DetBlock_TYPE&        first_block_type,
                                               uint32_t&             number_of_planes_of_the_same_type_in_first_block_of_the_diblock
                                              ) const{

    first_block_type = getBlockType(diblock_type, (uint32_t)0);/// calling base

    /// Get Number of planes of the same type in that first diblock.
    /// Needed to figure out which block current plane belongs to
    number_of_planes_of_the_same_type_in_first_block_of_the_diblock = getNumberOfPlaneTypeInBlock(first_block_type, plane_type);

    if(local_plane_in_diblock_in_view < number_of_planes_of_the_same_type_in_first_block_of_the_diblock) {
        output_block_type = first_block_type;
        return 0;
    }

    output_block_type = getBlockType(diblock_type, (uint32_t)1);
    return 1;
}



//------------------------------------------------------------------------------
offline_plane_t DAQChannelMapOLD::getLocalPlaneInDiblock(offline_plane_t local_plane_in_diblock_view,
                                                      DiBlock_TYPE diblock_type,
                                                      DetPlane_TYPE plane_type){
    DetBlock_TYPE   current_block_type;
    DetBlock_TYPE   first_block_type;
    uint32_t        number_of_planes_of_the_same_type_in_first_block_of_the_diblock;

    const uint32_t  local_block_in_diblock = getLocalBlockInDiblock(diblock_type, local_plane_in_diblock_view, plane_type, current_block_type,first_block_type,number_of_planes_of_the_same_type_in_first_block_of_the_diblock);

    const DetPlane_TYPE start_plane_type   = getStartPlaneTypeInBlock(current_block_type);

    /// Does the block starts with a different plane?
    uint32_t is_block_starts_with_different_plane = 0;
    if(plane_type != start_plane_type) is_block_starts_with_different_plane = 1;

    /// If it's the first block. We can calculate plane easily
    if(local_block_in_diblock == 0)
        return local_plane_in_diblock_view*2 + is_block_starts_with_different_plane;

    /// So, it's the second block. Need to do some magic

    /// To find local plane in the (second) block in the view we subtract
    /// number of planes of the same type in the first diblock from the local plane in diblock view
    const offline_plane_t local_plane_in_block_view = local_plane_in_diblock_view - number_of_planes_of_the_same_type_in_first_block_of_the_diblock;

    return getNumberOfPlanesInBlock(first_block_type) + local_plane_in_block_view*2 + is_block_starts_with_different_plane;
}

//------------------------------------------------------------------------------
online_cell_t DAQChannelMapOLD::getOnlineCell(int           detector,
                                           dcm_id_t      dcm_id,
                                           dcm_link_t    dcm_link,
                                           pixel_t       pixel,
                                           DiBlock_TYPE  diblock_type) const {
    const DetPlane_TYPE plane_type = getPlaneType(detector, dcm_id, diblock_type);
    return getOnlineCell(detector, dcm_id, dcm_link, pixel, diblock_type, plane_type);
}

//------------------------------------------------------------------------------
online_cell_t DAQChannelMapOLD::getOnlineCell(const int           detector,
                                           const dcm_id_t      dcm_id,
                                           const dcm_link_t    dcm_link,
                                           const pixel_t       pixel,
                                           const DiBlock_TYPE  diblock_type,
                                           const DetPlane_TYPE plane_type) const{

  const dcm_id_t      first_dcm_in_view   = getFirstDCMInView(detector, plane_type, diblock_type);
  const uint32_t      local_module_in_dcm = getLocalModuleInDCM(detector, dcm_id, dcm_link, diblock_type);
  
  online_cell_t cell = getNumberOfCellsPerDCM(diblock_type)*(dcm_id - first_dcm_in_view) + NUM_PIXELS*local_module_in_dcm + pixel;
  
  /// The NearDet has an uninstrumented section in DCM 01 in diblocks 1-3.  Nothing is plugged
  /// into FEBs 32-63, yet there is no gap in cell number.

  /// If we are in DCM 2, we have to subtract 32 due to this gap.
  if(detector == novadaq::cnv::kNEARDET && dcm_id == 2 && diblock_type != MC_ND_DIBLOCK)
    return cell - 32;

  /// If we are in the bottom half of DCM 1 (FEBs 32-63) then this is hardware with nothing plugged in,
  /// so return a BIG number.
  if(detector == novadaq::cnv::kNEARDET && dcm_id == 1 && diblock_type != MC_ND_DIBLOCK && dcm_link >= 32)
    return -1;
  
  // Otherwise, we do not have to do this special correction.
  return cell;
  
}

//------------------------------------------------------------------------------
online_cell_t DAQChannelMapOLD::getOnlineCell(const int           detector,
                                           const dcm_id_t      dcm_id,
                                           const dcm_link_t    dcm_link,
                                           const pixel_t       pixel,
                                           const DiBlock_TYPE  diblock_type,
                                           const DetPlane_TYPE plane_type,
                                           const uint32_t      number_of_x_dcms ) const{

  // WARNING:  This function allows you to override the constants defined in DAQCHannelMapConstants.h!!!

  const dcm_id_t      first_dcm_in_view   = getFirstDCMInView(plane_type, number_of_x_dcms);
  const uint32_t      local_module_in_dcm = getLocalModuleInDCM(detector, dcm_id, dcm_link, diblock_type);
  
  online_cell_t cell = getNumberOfCellsPerDCM(diblock_type)*(dcm_id - first_dcm_in_view) + NUM_PIXELS*local_module_in_dcm + pixel;
  
  /// The NearDet has an uninstrumented section in DCM 01 in diblocks 1-3.  Nothing is plugged
  /// into FEBs 32-63, yet there is no gap in cell number.

  /// If we are in DCM 2, we have to subtract 32 due to this gap.
  if(detector == novadaq::cnv::kNEARDET && dcm_id == 2 && diblock_type != MC_ND_DIBLOCK)
    return cell - 32;

  /// If we are in the bottom half of DCM 1 (FEBs 32-63) then this is hardware with nothing plugged in,
  /// so return a BIG number.
  if(detector == novadaq::cnv::kNEARDET && dcm_id == 1 && diblock_type != MC_ND_DIBLOCK && dcm_link >= 32)
    return -1;
  
  // Otherwise, we do not have to do this special correction.
  return cell;
  
}




//------------------------------------------------------------------------------
online_cell_t DAQChannelMapOLD::getOnlineCellOffline(lchan chan) const {

    const offline_cell_t cell_offline_0 = getCellOffline(chan);
    const DetPlane_TYPE  plane_type     = getPlaneTypeOffline(chan);
    const int            detector       = getDetectorOffline(chan);
    const DiBlock_TYPE   diblock_type   = getDiBlockTypeOffline(chan);

    return offlineCellToOnlineCell(cell_offline_0, detector, plane_type, diblock_type);
}

//------------------------------------------------------------------------------
offline_cell_t DAQChannelMapOLD::onlineCellToOfflineCell(online_cell_t online_cell_0,
                                                      int detector,
                                                      DetPlane_TYPE plane_type,
						      DiBlock_TYPE diblock_type
                                                      ) const {
  return ConvertCell((uint32_t)online_cell_0, detector, plane_type, diblock_type);
}
//------------------------------------------------------------------------------
online_cell_t DAQChannelMapOLD::offlineCellToOnlineCell(offline_cell_t offline_cell_0,
                                                     int detector,
                                                     DetPlane_TYPE plane_type,
						     DiBlock_TYPE diblock_type
                                                     ) const {
  return ConvertCell((uint32_t)offline_cell_0, detector, plane_type, diblock_type);
}

//------------------------------------------------------------------------------
uint32_t DAQChannelMapOLD::ConvertCell(uint32_t input_cell, 
                                    int detector,
                                    DetPlane_TYPE plane_type,
				    DiBlock_TYPE diblock_type
				    ) const {  
  
    /// If the plane is vertical, then the order is reversed
    if(plane_type == YZ_PLANE){
      // If this is the NearDet muon catcher, then there are only 64 cells (not 96)
      // in the YZ_PLANE.
      if(diblock_type == MC_ND_DIBLOCK)
	return 64 - input_cell -1;
      // Otherwise, we can use the default number of Y channels.
      return getNumberOfChannelsInYPlane(detector) - input_cell - 1;
    }// end of if the plane is horizontal

    /// If the plane is horizontal, then the order is the same
    return input_cell;
}

//------------------------------------------------------------------------------
bool DAQChannelMapOLD::print(dchan chan){

  const int   det      = getDetector(chan);
  const diblock_t       diblock  = getDiBlock(chan);
  const dcm_id_t        dcm_id   = getDCM(chan);
  const dcm_link_t      dcm_link = getDCMLink(chan);
  const pixel_t         pixel    = getPixel(chan);

  const offline_plane_t plane_0  = getPlane(chan);
  const offline_cell_t  cell_0   = getCell(chan);

  const lchan offline_channel = encodeLChan(det, plane_0, cell_0);

  std::cout<<"DAQChannel=";printHex(chan);
  std::cout<<"(";printBinary(chan);std::cout<<")";
  std::cout<<"\n";

  std::cout<<" Detector="<<det<<"("<<novadaq::cnv::DetInfo::GetName(det)<<")";
  std::cout<<" Diblock="<<diblock;
  std::cout<<" DCM="<<dcm_id;
  std::cout<<" DCMLink="<<dcm_link;
  std::cout<<" Pixel="<<pixel;

  std::cout<<"               ";

  std::cout<<" Plane="<<plane_0;
  std::cout<<" Cell="<<cell_0;

  std::cout<<"\n";
  std::cout<<" OfflineChannel=";
  printHex(offline_channel);std::cout<<"(";printBinary(offline_channel);std::cout<<")";

  std::cout<<"\n";

  return true;
}

//------------------------------------------------------------------------------
bool DAQChannelMapOLD::printOffline(lchan chan){

  const int   det      = getDetectorOffline(chan);
  const offline_plane_t plane_0  = getPlaneOffline(chan);
  const offline_cell_t  cell_0   = getCellOffline(chan);

  const diblock_t     diblock   = getDiBlockOffline(chan);
  const dcm_id_t      dcm_id    = getDCMOffline(chan);
  const dcm_link_t    dcm_link  = getDCMLinkOffline(chan);
  const apd_pixel_t   apd_pixel = pixel2APDPixel(getPixelOffline(chan));
  
  const dchan daq_channel   = encodeDChan(det, diblock, dcm_id, dcm_link, apd_pixel);

  std::cout<<"OfflineChannel=";printHex(chan);
  std::cout<<"(";printBinary(chan);std::cout<<")";
  std::cout<<"\n";

  std::cout<<"Detector="<<det<<"("<<novadaq::cnv::DetInfo::GetName(det)<<")";
  std::cout<<" Plane="<<plane_0;
  std::cout<<" Cell="<<cell_0;

  std::cout<<"               ";

  std::cout<<" Diblock="<<diblock;
  std::cout<<" DCM="<<dcm_id;
  std::cout<<" DCMLink="<<dcm_link;
  std::cout<<" Pixel="<<apd_pixel;

  std::cout<<"\n";
  std::cout<<"DAQChannel=";printHex(daq_channel);
  std::cout<<"(";printBinary(daq_channel);std::cout<<")";

  std::cout<<"\n";

  return true;
}

//------------------------------------------------------------------------------
// Stolen from RawDAQData::printWord
bool  DAQChannelMapOLD::printBinary(uint32_t value){

        const int SHIFT = 8 * sizeof( unsigned ) - 1;
        const unsigned MASK = 1 << SHIFT;

        for ( int i = 1; i <= SHIFT + 1; i++ ){
                std::cout << (  value & MASK ? '1' : '0' );
                value <<= 1;

                if ( i % 8 == 0 ) std::cout << ' ';
        }
        
        return true;
}// end of printBinary

//------------------------------------------------------------------------------
// Stolen from RawDAQData::printWord
bool DAQChannelMapOLD::printHex(uint32_t value){
    printf("0x%08x",value);

    return true;
}// end of printHex


