#include "DAQDataFormats/RawConfigurationHeaderV0.h"
#include "DAQDataFormats/RawConfigurationTail.h"
#include "DAQDataFormats/RawConfigurationSystemID.h"
#include "DAQDataFormats/BitFields.h"
#include <boost/format.hpp>

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawConfigurationHeader
#define  CURRENT_CLASS_VERSION        0
#define  CURRENT_FIELDS_SUB_NAMESPACE rawconfhdr
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats{
namespace VERSION_NAMESPACE{

//------------------------------------------------------------------------------
// Default constructor
RawConfigurationHeader::RawConfigurationHeader(const version_t version):
             PREVIOUS_VERSION_CLASS_NAME(version)
            ,ZEROOUT_GET_AND_SET(ConfigurationHeaderVersion)
            ,ZEROOUT_GET_AND_SET(MarkerHi1)
            ,ZEROOUT_GET_AND_SET(MarkerHi2)
            ,ZEROOUT_GET_AND_SET(DetectorID            )
            ,ZEROOUT_GET_AND_SET(ConfigurationBlockSize)
            ,ZEROOUT_GET_AND_SET(ResourceManagerID     )
            ,ZEROOUT_GET_AND_SET(RunNumber             )
            ,ZEROOUT_GET_AND_SET(RunType               )
            ,ZEROOUT_GET_AND_SET(ConfigurationIDX      )
            ,ZEROOUT_GET_AND_SET(Partition             )
            ,ZEROOUT_GET_AND_SET(Subrun                )
            ,ZEROOUT_GET_AND_SET(SystemNumber          )
            ,ZEROOUT_GET_AND_SET(SystemVersionIDX      )
            ,ZEROOUT_FUNCTION(advanceSystemNumber)
{}

RawConfigurationHeader::RawConfigurationHeader (const RawConfigurationHeader& copyin):
             PREVIOUS_VERSION_CLASS_NAME(copyin)
            ,DEEPCOPY_GET_AND_SET(ConfigurationHeaderVersion)
            ,DEEPCOPY_GET_AND_SET(MarkerHi1)
            ,DEEPCOPY_GET_AND_SET(MarkerHi2)
            ,DEEPCOPY_GET_AND_SET(DetectorID            )
            ,DEEPCOPY_GET_AND_SET(ConfigurationBlockSize)
            ,DEEPCOPY_GET_AND_SET(ResourceManagerID     )
            ,DEEPCOPY_GET_AND_SET(RunNumber             )
            ,DEEPCOPY_GET_AND_SET(RunType               )
            ,DEEPCOPY_GET_AND_SET(ConfigurationIDX      )
            ,DEEPCOPY_GET_AND_SET(Partition             )
            ,DEEPCOPY_GET_AND_SET(Subrun                )
            ,DEEPCOPY_GET_AND_SET(SystemNumber          )
            ,DEEPCOPY_GET_AND_SET(SystemVersionIDX      )
            ,DEEPCOPY_FUNCTION(advanceSystemNumber)
{}
//------------------------------------------------------------------------------
// Destructor
RawConfigurationHeader::~RawConfigurationHeader() { }

//------------------------------------------------------------------------------
// Initialization of data members
void RawConfigurationHeader::init(){
    
    EXECUTE_FUNCTION(setConfigurationHeaderVersion)(_version);

    // Set the Marker
    EXECUTE_FUNCTION(setMarkerHi1)(daqdataformats::rawconfhdr::Marker_value[0]);
    EXECUTE_FUNCTION(setMarkerHi2)(daqdataformats::rawconfhdr::Marker_value[1]);
}

//------------------------------------------------------------------------------
bool RawConfigurationHeader::advanceSystemNumber() { RETURN_FUNCTION(setSystemNumber)(EXECUTE_FUNCTION(getSystemNumber)() + 1);}

//------------------------------------------------------------------------------
// Printing the rawconfigurationheader structure
void RawConfigurationHeader::print(std::ostream& os) const{
    os<<boost::format("\n---RawConfigurationHeader---------------------------------------------size = %u words\n")
        % EXECUTE_FUNCTION(sizeofdata)();

    os<<boost::format(" MarkerHi1=%u"                 ) % EXECUTE_FUNCTION(getMarkerHi1)();
    os<<boost::format(" MarkerHi2=%u"                 ) % EXECUTE_FUNCTION(getMarkerHi2)();
    os<<boost::format(" ConfigurationHeaderVersion=%u") % EXECUTE_FUNCTION(getConfigurationHeaderVersion)();
    os<<boost::format(" DetectorID=%u"                ) % EXECUTE_FUNCTION(getDetectorID)();
    os<<boost::format(" ConfigurationBlockSize=%u"    ) % EXECUTE_FUNCTION(getConfigurationBlockSize)();
    os<<boost::format(" ResourceManagerID=%u"         ) % EXECUTE_FUNCTION(getResourceManagerID)();
    os<<boost::format(" RunNumber=%u"                 ) % EXECUTE_FUNCTION(getRunNumber)();
    os<<boost::format(" RunType=%u"                   ) % EXECUTE_FUNCTION(getRunType)();
    os<<boost::format(" ConfigurationIDX=%u"          ) % EXECUTE_FUNCTION(getConfigurationIDX)();
    os<<boost::format(" Partition=%u"                 ) % EXECUTE_FUNCTION(getPartition)();
    os<<boost::format(" Subrun=%u"                    ) % EXECUTE_FUNCTION(getSubrun)();
    os<<boost::format(" SystemNumber=%u"              ) % EXECUTE_FUNCTION(getSystemNumber)();
    os<<boost::format(" SystemVersionIDX=%u"          ) % EXECUTE_FUNCTION(getSystemVersionIDX)();
}

}}// end of namespaces