#include "DAQDataFormats/RawConfigurationName.h"
#include "DAQDataFormats/DataFormatException.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawConfigurationName
#define  LAST_CLASS_VERSION           0
#define  CURRENT_FIELDS_SUB_NAMESPACE rawconfname
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats{

//------------------------------------------------------------------------------
// Default constructor
RawConfigurationName::RawConfigurationName(const version_t version):
    PREVIOUS_VERSION_CLASS_NAME(version)
{
    INITIALIZE_DEBUGGING_VARIABLES;
    reserveInternalBuffer(getPredefinedSize());
    defaultConstructorFloatingSize();
}
//------------------------------------------------------------------------------
RawConfigurationName::RawConfigurationName(const RawConfigurationName& copy_in):
    PREVIOUS_VERSION_CLASS_NAME(copy_in)
{
    INITIALIZE_DEBUGGING_VARIABLES;
    lastOperationsInDeepCopy(copy_in);
}

//------------------------------------------------------------------------------
RawConfigurationName::RawConfigurationName(const void* buffer, const version_t version):
    PREVIOUS_VERSION_CLASS_NAME(version)
{
    INITIALIZE_DEBUGGING_VARIABLES;
    copy(buffer);
}

//------------------------------------------------------------------------------
// Destructor
RawConfigurationName::~RawConfigurationName() { }


/// All binding routines
#define STANDARD_ROUTINES                                                                                \
            BIND_GET_AND_SET    (Version   , version0, uint32_t);                                        \
            BIND_GET_AND_SET    (Size      , version0, uint32_t);                                        \
            BIND_GET            (HeaderSize, version0, uint32_t);                                        \
            BIND_CUSTOM_FUNCTION(getName   , version0, const, uint32_t, std::string&);                   \
            BIND_CUSTOM_FUNCTION(setName   , version0,      , uint32_t, const std::string&);             \
            BIND_CUSTOM_FUNCTION(getChar1  , version0, const, uint32_t, const uint32_t);                 \
            BIND_CUSTOM_FUNCTION(getChar2  , version0, const, uint32_t, const uint32_t);                 \
            BIND_CUSTOM_FUNCTION(getChar3  , version0, const, uint32_t, const uint32_t);                 \
            BIND_CUSTOM_FUNCTION(getChar4  , version0, const, uint32_t, const uint32_t);                 \
            BIND_CUSTOM_FUNCTION(setChar1  , version0,      , bool    , const uint32_t, const uint32_t); \
            BIND_CUSTOM_FUNCTION(setChar2  , version0,      , bool    , const uint32_t, const uint32_t); \
            BIND_CUSTOM_FUNCTION(setChar3  , version0,      , bool    , const uint32_t, const uint32_t); \
            BIND_CUSTOM_FUNCTION(setChar4  , version0,      , bool    , const uint32_t, const uint32_t); \
            BIND_STADARD_FUNCTIONS;
            

#include "DAQDataFormats/setFunctionPointers_template.inc"


}// end of namespace