#include "DAQDataFormats/RawMicroSliceHeader.h"
#include "DAQDataFormats/DataFormatException.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawMicroSliceHeader
#define  LAST_CLASS_VERSION           1
#define  CURRENT_FIELDS_SUB_NAMESPACE rawmuhdr
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats {

//------------------------------------------------------------------------------
// Default constructor
RawMicroSliceHeader::RawMicroSliceHeader(const version_t version):
    PREVIOUS_VERSION_CLASS_NAME(version)
{
    _function_sizeofdata = (sizeofdata_t) &RawMicroSliceHeader::sizeofdata;
    INITIALIZE_DEBUGGING_VARIABLES;
    /// Apply the default constructor
    defaultConstructorFixedSize();
}

//------------------------------------------------------------------------------
RawMicroSliceHeader::RawMicroSliceHeader(const RawMicroSliceHeader& copy_in):
    PREVIOUS_VERSION_CLASS_NAME(copy_in)
{
    INITIALIZE_DEBUGGING_VARIABLES;
    lastOperationsInDeepCopy(copy_in);
}

//------------------------------------------------------------------------------
RawMicroSliceHeader::RawMicroSliceHeader(const void* microsliceheader_buffer, const version_t version):
    PREVIOUS_VERSION_CLASS_NAME(version)
{
    _function_sizeofdata = (sizeofdata_t) &RawMicroSliceHeader::sizeofdata;
    copy(microsliceheader_buffer);
}

//------------------------------------------------------------------------------
// Destructor
RawMicroSliceHeader::~RawMicroSliceHeader() { }

//------------------------------------------------------------------------------
 bool RawMicroSliceHeader::setVersion(const version_t ver)
 {

    if(ver != rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION){
        version0::RawMicroSliceHeader::setVersion(ver);
        _version = ver;
        return setFunctionPointers();
    }

    _version = ver;
    return true;
 }
 
//------------------------------------------------------------------------------
bool RawMicroSliceHeader::setVersionSoft(const version_t ver)
{
        _version = ver;
        return setFunctionPointers();
}

/// All binding routines
/// Not binding size of data, since it's fixed to 1
#define STANDARD_ROUTINES                                \
            BIND_GET_AND_SET(Extra , version0, uint32_t);\
            BIND_GET_AND_SET(Status, version0, uint32_t);\
            BIND_STADARD_FUNCTIONS;

#include "DAQDataFormats/setFunctionPointers_template.inc"




}// end of namespace