#include <DatabaseUtils/DAQConfig/PixelThresholds.h>
#include <DatabaseUtils/DAQConfig/DBColumnParams.h>
#include <DatabaseUtils/DAQConfig/NamedConfigUtils.h>
#include <DatabaseUtils/DAQConfig/SubsystemNameSpec.h>
#include <algorithm>

namespace dbutils {
namespace daqconfig {

PixelThresholds::
PixelThresholds(const int64_t& globalConfigId,
                std::vector<std::string> dcmList)
{
    IDSpec idSpec(globalConfigId, IDSpec::GLOBAL, IDSpec::DAQ);
    std::vector<int64_t> subsystemConfigIdList;
    bool status = NamedConfigUtils::
        getSubsystemConfigIdAncestryFromGlobalId(idSpec,
                                                 SubsystemNameSpec::
                                                 DCM_SUBSYSTEM_NAME,
                                                 SubsystemNameSpec::
                                                 HARDWARE_CONFIG_TYPE,
                                                 subsystemConfigIdList);
    if (! status) {return;}
    std::sort(dcmList.begin(), dcmList.end());

    {
        ConfigDataSelector cfgSelector;
        cfgSelector.addSelectionColumn(DBColumnParams::DCMID_COLUMN_NAME,
                                       dcmList[0], dcmList[dcmList.size()-1]);
        cfgSelector.addSelectionColumn(DBColumnParams::FEBID_COLUMN_NAME);
        cfgSelector.addSelectionColumn(DBColumnParams::PIXELID_COLUMN_NAME);

        std::vector<std::string> dataColumnsOfInterest;
        dataColumnsOfInterest.push_back("threshold");

        std::string fname("NovaDatabase/tables/DAQConfig/PixelThresholds.xml");
        _pixelThresholdDataSet.
            reset(new ConfigDataTree(fname, subsystemConfigIdList, cfgSelector,
                                     dataColumnsOfInterest));
    }

    _hasValidData = _pixelThresholdDataSet.get() != 0 &&
        _pixelThresholdDataSet->hasValidData();
}

bool PixelThresholds::
getThreshold(const std::string& dcmName, const int& febId,
             const int& pixelId, int16_t& threshold)
{
    if (_pixelThresholdDataSet.get() == 0) {return false;}
    return _pixelThresholdDataSet->
        getEffectiveValueFor3Keys(dcmName, febId, pixelId,
                                  "threshold", threshold);
}

bool PixelThresholds::
getThresholdsForFEB(const std::string& dcmName, const int& febId,
                    std::vector<int16_t>& thresholdList)
{
    if (_pixelThresholdDataSet.get() == 0) {return false;}

    thresholdList.clear();
    for (int ipix = 0; ipix < 32; ++ipix) {
        int16_t threshold = 0;
        if (! _pixelThresholdDataSet->
            getEffectiveValueFor3Keys(dcmName, febId, ipix,
                                      "threshold", threshold)) {
            return false;
        }
        thresholdList.push_back(threshold);
    }

    return true;
}

};  // end of namespace daqconfig
};  // end of namespace dbutils
