#include <NovaDAQUtilities/TimedCachePolicy.h>

namespace novadaq {

/**
 * Constructor.
 *
 * @param timeoutInMinutes The time, in minutes, after which the 
 *        policy will report that a cache has expired.
 */
TimedCachePolicy::TimedCachePolicy(int timeoutInMinutes) :
    _timeoutInSeconds(timeoutInMinutes * 60) {}

/**
 * Tells the policy that a cache update was done.  This is used to
 * reset the state of the policy to "unexpired" after an update.
 */
void TimedCachePolicy::_markAsUpdated() {
    _lastUpdateTime = time(0);
}

/**
 * Tests if the associated cache has expired based on the criteria
 * in the policy.
 *
 * @returns true if the cache has expired, false if not.
 */
bool TimedCachePolicy::_checkIfExpired() {
    time_t now = time(0);
    time_t cacheAge = now - _lastUpdateTime;
    return (cacheAge >= _timeoutInSeconds);
}

};  // end of namespace novadaq
